@@ -4,10 +4,15 @@ Thse elements definitions appear in https://dx.doi.org/10.1137/S0036142903431924
 (Arnold, Boffi, Falk, 2005)
 """
 
+import typing
+
 from ..finite_element import CiarletElement
-from ..functionals import NormalIntegralMoment, IntegralMoment, IntegralOfDivergenceAgainst
+from ..functionals import (IntegralMoment, IntegralOfDivergenceAgainst, ListOfFunctionals,
+                           NormalIntegralMoment)
+from ..functions import FunctionInput
 from ..moments import make_integral_moment_dofs
-from ..symbolic import x
+from ..references import Reference
+from ..symbols import x
 from .lagrange import Lagrange
 from .q import Nedelec
 
@@ -15,14 +20,23 @@ from .q import Nedelec
 class ArnoldBoffiFalk(CiarletElement):
     """An Arnold-Boffi-Falk element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
         assert reference.name == "quadrilateral"
-        poly = [(x[0] ** i * x[1] ** j, 0)
-                for i in range(order + 3) for j in range(order + 1)]
+        poly: typing.List[FunctionInput] = []
+        poly += [
+            (x[0] ** i * x[1] ** j, 0)
+            for i in range(order + 3) for j in range(order + 1)]
         poly += [(0, x[0] ** i * x[1] ** j)
                  for i in range(order + 1) for j in range(order + 3)]
 
-        dofs = make_integral_moment_dofs(
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             edges=(NormalIntegralMoment, Lagrange, order, {"variant": variant}),
             faces=(IntegralMoment, Nedelec, order, {"variant": variant})
@@ -37,9 +51,14 @@ class ArnoldBoffiFalk(CiarletElement):
                 entity=(2, 0), mapping="contravariant"))
 
         super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
+        self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["Arnold-Boffi-Falk", "ABF"]
