@@ -4,22 +4,30 @@ This element's definition appears in https://doi.org/10.1017/S000192400008489X
 (Arygris, Fried, Scharpf, 1968)
 """
 
+import typing
+
 from ..finite_element import CiarletElement
-from ..polynomials import polynomial_set
-from ..functionals import (PointEvaluation, PointDirectionalDerivativeEvaluation,
-                           PointNormalDerivativeEvaluation,
-                           PointComponentSecondDerivativeEvaluation)
-from ..symbolic import sym_sum
+from ..functionals import (ListOfFunctionals, PointComponentSecondDerivativeEvaluation,
+                           PointDirectionalDerivativeEvaluation, PointEvaluation,
+                           PointNormalDerivativeEvaluation)
+from ..functions import FunctionInput
+from ..polynomials import polynomial_set_1d
+from ..references import Reference
 
 
 class Argyris(CiarletElement):
     """Argyris finite element."""
 
-    def __init__(self, reference, order):
-        from symfem import create_reference
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+        """
         assert order == 5
         assert reference.name == "triangle"
-        dofs = []
+        dofs: ListOfFunctionals = []
         for v_n, vs in enumerate(reference.vertices):
             dofs.append(PointEvaluation(reference, vs, entity=(0, v_n)))
             for i in range(reference.tdim):
@@ -30,18 +38,15 @@ class Argyris(CiarletElement):
                 for j in range(i + 1):
                     dofs.append(PointComponentSecondDerivativeEvaluation(
                         reference, vs, (i, j), entity=(0, v_n)))
-        for e_n, vs in enumerate(reference.sub_entities(1)):
-            sub_ref = create_reference(
-                reference.sub_entity_types[1],
-                vertices=[reference.vertices[v] for v in vs])
-            midpoint = tuple(sym_sum(i) / len(i)
-                             for i in zip(*[reference.vertices[i] for i in vs]))
+        for e_n in range(reference.sub_entity_count(1)):
+            assert isinstance(reference.sub_entity_types[1], str)
+            sub_ref = reference.sub_entity(1, e_n)
             dofs.append(PointNormalDerivativeEvaluation(
-                reference, midpoint, sub_ref, entity=(1, e_n)))
+                reference, sub_ref.midpoint(), sub_ref, entity=(1, e_n)))
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_1d(reference.tdim, order)
 
-        super().__init__(
-            reference, order, polynomial_set(reference.tdim, 1, order), dofs, reference.tdim, 1
-        )
+        super().__init__(reference, order, poly, dofs, reference.tdim, 1)
 
     names = ["Argyris"]
     references = ["triangle"]
