@@ -4,32 +4,45 @@ This element's definition appears in https://doi.org/10.2307/2007793
 (Bernardi and Raugel, 1985)
 """
 
+import typing
+
 import sympy
+
 from ..finite_element import CiarletElement
+from ..functionals import (DivergenceIntegralMoment, DotPointEvaluation, ListOfFunctionals,
+                           NormalIntegralMoment)
+from ..functions import FunctionInput
 from ..moments import make_integral_moment_dofs
-from ..polynomials import polynomial_set
-from ..functionals import (NormalIntegralMoment, DotPointEvaluation,
-                           DivergenceIntegralMoment)
-from ..symbolic import x
+from ..polynomials import polynomial_set_vector
+from ..references import Reference
+from ..symbols import x
 from .lagrange import Lagrange
 
 
 class BernardiRaugel(CiarletElement):
     """Bernardi-Raugel Hdiv finite element."""
 
-    def __init__(self, reference, order):
-        poly = polynomial_set(reference.tdim, reference.tdim, order)
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_vector(reference.tdim, reference.tdim, order)
 
         p = Lagrange(reference, 1, variant="equispaced")
 
         for i in range(reference.sub_entity_count(reference.tdim - 1)):
             sub_e = reference.sub_entity(reference.tdim - 1, i)
-            bubble = 1
+            bubble = sympy.Integer(1)
             for j in reference.sub_entities(reference.tdim - 1)[i]:
                 bubble *= p.get_basis_function(j)
             poly.append(tuple(bubble * j for j in sub_e.normal()))
 
-        dofs = []
+        dofs: ListOfFunctionals = []
 
         for n in range(reference.sub_entity_count(reference.tdim - 1)):
             facet = reference.sub_entity(reference.tdim - 1, n)
@@ -48,10 +61,9 @@ class BernardiRaugel(CiarletElement):
             assert order == 2 and reference.name == "tetrahedron"
 
             for i in range(reference.tdim):
+                bf = p.get_basis_functions()
                 poly.append(tuple(
-                    p.get_basis_function(0) * p.get_basis_function(1)
-                    * p.get_basis_function(2) * p.get_basis_function(3)
-                    if j == i else 0 for j in range(reference.tdim)))
+                    bf[0] * bf[1] * bf[2] * bf[3] if j == i else 0 for j in range(reference.tdim)))
 
             for e_n, edge in enumerate(reference.edges):
                 v1 = reference.vertices[edge[0]]
@@ -62,10 +74,11 @@ class BernardiRaugel(CiarletElement):
                                                mapping="contravariant"))
             for f_n in range(reference.sub_entity_count(2)):
                 face = reference.sub_entity(2, f_n)
-                normal = [i * face.jacobian() for i in face.normal()]
+                normal = tuple(i * face.jacobian() for i in face.normal())
                 for e_n in range(3):
-                    edge = face.sub_entity(1, e_n)
-                    midpoint = tuple(sympy.Rational(i + j, 2) for i, j in zip(*edge.vertices))
+                    edge_entity = face.sub_entity(1, e_n)
+                    midpoint = tuple(
+                        sympy.Rational(i + j, 2) for i, j in zip(*edge_entity.vertices))
                     dofs.append(DotPointEvaluation(
                         reference, midpoint, normal, entity=(2, f_n), mapping="contravariant"))
 
