@@ -5,17 +5,22 @@ https://doi.org/10.1007/s00211-010-0327-2 (Kirby, 2011) and
 https://doi.org/10.1137/11082539X (Ainsworth, Andriamaro, Davydov, 2011)
 """
 
+import typing
+
 import sympy
-from ..symbolic import x, t, subs, to_sympy
+
 from ..finite_element import CiarletElement
-from ..polynomials import polynomial_set
-from ..functionals import BaseFunctional, PointEvaluation
-from ..polynomials import orthogonal_basis
+from ..functionals import BaseFunctional, ListOfFunctionals, PointEvaluation
+from ..functions import AnyFunction, FunctionInput
+from ..geometry import PointType
+from ..polynomials import orthogonal_basis, polynomial_set_1d
+from ..references import Reference
+from ..symbols import AxisVariablesNotSingle, t, x
 
 
-def single_choose(n, k):
+def single_choose(n: int, k: int) -> sympy.core.expr.Expr:
     """Calculate choose function of a set of powers."""
-    out = to_sympy(1)
+    out = sympy.Integer(1)
     for i in range(k + 1, n + 1):
         out *= i
     for i in range(1, n - k + 1):
@@ -23,27 +28,24 @@ def single_choose(n, k):
     return out
 
 
-def choose(n, powers):
+def choose(n: int, powers: typing.List[int]) -> sympy.core.expr.Expr:
     """Calculate choose function of a set of powers."""
-    out = to_sympy(1)
+    out = sympy.Integer(1)
     for p in powers:
         out *= single_choose(n, p)
         n -= p
     return out
 
 
-def bernstein_polynomials(n, d, vars=x):
-    """
-    Return a list of Bernstein polynomials.
+def bernstein_polynomials(
+    n: int, d: int, vars: AxisVariablesNotSingle = x
+) -> typing.List[sympy.core.expr.Expr]:
+    """Return a list of Bernstein polynomials.
 
-    Parameters
-    ----------
-    n : int
-        The polynomial order
-    d : int
-        The topological dimension
-    vars: list
-        The variables to use
+    Args:
+        n: The polynomial order
+        d: The topological dimension
+        vars: The variables to use
     """
     poly = []
     if d == 1:
@@ -73,10 +75,20 @@ def bernstein_polynomials(n, d, vars=x):
 class BernsteinFunctional(BaseFunctional):
     """Functional for a Bernstein element."""
 
-    def __init__(self, reference, integral_domain, index, degree, entity):
+    def __init__(self, reference: Reference, integral_domain: Reference, index: int,
+                 degree: int, entity: typing.Tuple[int, int]):
+        """Create the functional.
+
+        Args:
+            reference: The reference element
+            integral_domain: The subentity to integrate over
+            index: The index of the bernstein polynomial
+            degree: The polynomial degree
+            entity: The entity this functional is associated with
+        """
         super().__init__(reference, entity, "identity")
-        self.orth = [
-            o / sympy.sqrt(integral_domain.integral(o * o))
+        orth = [
+            o / sympy.sqrt((o * o).integral(integral_domain))
             for o in orthogonal_basis(integral_domain.name, degree, 0, t[:integral_domain.tdim])[0]
         ]
         self.ref = integral_domain
@@ -85,32 +97,27 @@ class BernsteinFunctional(BaseFunctional):
 
         bern = bernstein_polynomials(degree, integral_domain.tdim, t)
         mat = sympy.Matrix(
-            [[integral_domain.integral(o * b) for b in bern] for o in self.orth])
+            [[(o * b).integral(integral_domain) for b in bern] for o in orth])
         minv = mat.inv()
-        self.alpha = minv.row(index)
+        alpha = minv.row(index)
 
-    def dof_point(self):
+        self.moment = sum(i * j for i, j in zip(alpha, orth))
+
+    def dof_point(self) -> PointType:
         """Get the location of the DOF in the cell."""
         return self.ref.sub_entity(*self.entity).midpoint()
 
-    def dof_direction(self):
-        """Get the direction of the DOF."""
-        return None
-
-    def eval(self, function, symbolic=True):
+    def _eval_symbolic(self, function: AnyFunction) -> AnyFunction:
         """Apply the functional to a function."""
         point = [i for i in self.ref.origin]
         for i, a in enumerate(zip(*self.ref.axes)):
             for j, k in zip(a, t):
                 point[i] += j * k
 
-        coeffs = [
-            self.ref.integral(subs(function, x, point) * f)
-            for f in self.orth
-        ]
-        return sum(i * j for i, j in zip(self.alpha, coeffs))
+        integrand = function.subs(x, point) * self.moment
+        return integrand.integral(self.ref)
 
-    def get_tex(self):
+    def get_tex(self) -> typing.Tuple[str, typing.List[str]]:
         """Get a representation of the functional as TeX, and list of terms involved."""
         if self.reference.tdim == self.ref.tdim:
             return f"v\\mapsto c_{{{self.index}}}", [
@@ -131,20 +138,29 @@ class BernsteinFunctional(BaseFunctional):
 class Bernstein(CiarletElement):
     """Bernstein finite element."""
 
-    def __init__(self, reference, order):
-        poly = polynomial_set(reference.tdim, 1, order)
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
 
+        Args:
+            reference: The reference element
+            order: The polynomial order
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_1d(reference.tdim, order)
+
+        dofs: ListOfFunctionals = []
         if order == 0:
-            dofs = [PointEvaluation(reference, reference.midpoint(), (reference.tdim, 0))]
+            dofs = [
+                PointEvaluation(reference, reference.midpoint(), (reference.tdim, 0))]
         else:
-            def index(x, y=0, z=0):
+            def index(x: int, y: int = 0, z: int = 0) -> int:
+                """Compute the 1D index."""
                 return (
                     z * (z ** 2 - 3 * z * order - 6 * z + 3 * order ** 2 + 12 * order + 11) // 6
                     + y * (2 * (order - z) + 3 - y) // 2
                     + x
                 )
 
-            dofs = []
             for vn, v in enumerate(reference.vertices):
                 dofs.append(PointEvaluation(reference, v, (0, vn)))
 
