@@ -4,20 +4,30 @@ This element's definition appears in https://doi.org/10.1017/S0001925900004546
 (Morley, 1968)
 """
 
+import typing
+
 from ..finite_element import CiarletElement
-from ..polynomials import polynomial_set
-from ..functionals import PointEvaluation, PointNormalDerivativeEvaluation
+from ..functionals import ListOfFunctionals, PointEvaluation, PointNormalDerivativeEvaluation
+from ..functions import FunctionInput
+from ..polynomials import polynomial_set_1d
+from ..references import Reference
 
 
 class Morley(CiarletElement):
     """Morley finite element."""
 
-    def __init__(self, reference, order):
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+        """
         if reference.vertices != reference.reference_vertices:
             raise NotImplementedError()
         assert order == 2
         assert reference.name == "triangle"
-        dofs = []
+        dofs: ListOfFunctionals = []
         for v_n, vs in enumerate(reference.vertices):
             dofs.append(PointEvaluation(reference, vs, entity=(0, v_n)))
         for e_n in range(reference.sub_entity_count(1)):
@@ -26,9 +36,10 @@ class Morley(CiarletElement):
             dofs.append(
                 PointNormalDerivativeEvaluation(reference, midpoint, sub_ref, entity=(1, e_n)))
 
-        super().__init__(
-            reference, order, polynomial_set(reference.tdim, 1, order), dofs, reference.tdim, 1
-        )
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_1d(reference.tdim, order)
+
+        super().__init__(reference, order, poly, dofs, reference.tdim, 1)
 
     names = ["Morley"]
     references = ["triangle"]
