@@ -4,20 +4,31 @@ This element's definition appears in https://doi.org/10.1090/S0025-5718-2012-026
 (Wang, Xu, 2013)
 """
 
+import typing
+
 from ..finite_element import CiarletElement
-from ..polynomials import polynomial_set
-from ..functionals import (PointEvaluation, IntegralOfDirectionalMultiderivative,
-                           IntegralAgainst)
+from ..functionals import (IntegralAgainst, IntegralOfDirectionalMultiderivative, ListOfFunctionals,
+                           PointEvaluation)
+from ..functions import FunctionInput
+from ..polynomials import polynomial_set_1d
+from ..references import Reference
 
 
 class MorleyWangXu(CiarletElement):
     """Morley-Wang-Xu finite element."""
 
-    def __init__(self, reference, order):
-        assert order <= reference.tdim
-        poly = polynomial_set(reference.tdim, 1, order)
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
 
-        dofs = []
+        Args:
+            reference: The reference element
+            order: The polynomial order
+        """
+        assert order <= reference.tdim
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_1d(reference.tdim, order)
+
+        dofs: ListOfFunctionals = []
 
         if order == 1:
             if reference.tdim == 1:
@@ -59,7 +70,7 @@ class MorleyWangXu(CiarletElement):
                         normals.append(face.normal())
                 for orders in [(1, 0), (0, 1)]:
                     dofs.append(IntegralOfDirectionalMultiderivative(
-                        reference, subentity, normals, orders, (1, e_n),
+                        reference, subentity, tuple(normals), orders, (1, e_n),
                         scale=1 / volume))
             for f_n, vs in enumerate(reference.sub_entities(2)):
                 subentity = reference.sub_entity(2, f_n)
