@@ -4,10 +4,14 @@ These elements' definitions appear in https://doi.org/10.1007/BF01396415
 (Nedelec, 1980) and https://doi.org/10.1007/BF01389668 (Nedelec, 1986)
 """
 
+import typing
+
 from ..finite_element import CiarletElement
+from ..functionals import IntegralMoment, ListOfFunctionals, TangentIntegralMoment
+from ..functions import FunctionInput
 from ..moments import make_integral_moment_dofs
-from ..polynomials import polynomial_set, Hcurl_polynomials
-from ..functionals import TangentIntegralMoment, IntegralMoment
+from ..polynomials import Hcurl_polynomials, polynomial_set_vector
+from ..references import Reference
 from .lagrange import Lagrange, VectorLagrange
 from .rt import RaviartThomas
 
@@ -15,10 +19,18 @@ from .rt import RaviartThomas
 class NedelecFirstKind(CiarletElement):
     """Nedelec first kind Hcurl finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = polynomial_set(reference.tdim, reference.tdim, order - 1)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_vector(reference.tdim, reference.tdim, order - 1)
         poly += Hcurl_polynomials(reference.tdim, reference.tdim, order)
-        dofs = make_integral_moment_dofs(
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             edges=(TangentIntegralMoment, Lagrange, order - 1,
                    {"variant": variant}),
@@ -31,8 +43,12 @@ class NedelecFirstKind(CiarletElement):
         super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["Nedelec", "Nedelec1", "N1curl"]
@@ -44,10 +60,18 @@ class NedelecFirstKind(CiarletElement):
 class NedelecSecondKind(CiarletElement):
     """Nedelec second kind Hcurl finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = polynomial_set(reference.tdim, reference.tdim, order)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
 
-        dofs = make_integral_moment_dofs(
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_vector(reference.tdim, reference.tdim, order)
+
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             edges=(TangentIntegralMoment, Lagrange, order, {"variant": variant}),
             faces=(IntegralMoment, RaviartThomas, order - 1, "covariant", {"variant": variant}),
@@ -57,8 +81,12 @@ class NedelecSecondKind(CiarletElement):
         super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["Nedelec2", "N2curl"]
