@@ -1,11 +1,14 @@
 """Nedelec elements on prisms."""
 
+import typing
+
 from ..finite_element import CiarletElement
+from ..functionals import IntegralAgainst, IntegralMoment, ListOfFunctionals, TangentIntegralMoment
+from ..functions import FunctionInput
 from ..moments import make_integral_moment_dofs
-from ..polynomials import (Hcurl_polynomials, polynomial_set_1d,
-                           polynomial_set)
-from ..functionals import TangentIntegralMoment, IntegralMoment, IntegralAgainst
-from ..symbolic import x
+from ..polynomials import Hcurl_polynomials, polynomial_set_1d, polynomial_set_vector
+from ..references import Reference
+from ..symbols import x
 from .lagrange import Lagrange, VectorLagrange
 from .q import RaviartThomas as QRT
 
@@ -13,17 +16,26 @@ from .q import RaviartThomas as QRT
 class Nedelec(CiarletElement):
     """Nedelec Hcurl finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
         from .. import create_reference
 
-        poly = [(i[0] * j, i[1] * j, 0)
-                for i in polynomial_set(2, 2, order - 1) + Hcurl_polynomials(2, 2, order)
-                for j in polynomial_set_1d(1, order, x[2:])]
+        poly: typing.List[FunctionInput] = []
+        poly += [
+            (i[0] * j, i[1] * j, 0)
+            for i in polynomial_set_vector(2, 2, order - 1) + Hcurl_polynomials(2, 2, order)
+            for j in polynomial_set_1d(1, order, x[2:])]
         poly += [(0, 0, i * j)
                  for i in polynomial_set_1d(2, order, x[:2])
                  for j in polynomial_set_1d(1, order - 1, x[2:])]
 
-        dofs = make_integral_moment_dofs(
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             edges=(TangentIntegralMoment, Lagrange, order - 1,
                    {"variant": variant}),
@@ -54,8 +66,12 @@ class Nedelec(CiarletElement):
         super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["Nedelec", "Ncurl"]
