@@ -1,37 +1,45 @@
 """Q elements on tensor product cells."""
 
-import sympy
+import typing
 from itertools import product
-from ..finite_element import CiarletElement
+
+import sympy
+
+from ..finite_element import CiarletElement, FiniteElement
+from ..functionals import (DotPointEvaluation, IntegralMoment, ListOfFunctionals,
+                           NormalIntegralMoment, PointEvaluation, TangentIntegralMoment)
+from ..functions import FunctionInput
 from ..moments import make_integral_moment_dofs
-from ..polynomials import quolynomial_set, Hdiv_quolynomials, Hcurl_quolynomials
+from ..polynomials import (Hcurl_quolynomials, Hdiv_quolynomials, quolynomial_set_1d,
+                           quolynomial_set_vector)
 from ..quadrature import get_quadrature
-from ..functionals import (PointEvaluation, DotPointEvaluation, IntegralMoment,
-                           TangentIntegralMoment, NormalIntegralMoment)
+from ..references import Reference
 
 
 class Q(CiarletElement):
     """A Q element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        from symfem import create_reference
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
 
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        dofs: ListOfFunctionals = []
         if order == 0:
-            dofs = [
-                PointEvaluation(
-                    reference, tuple(sympy.Rational(1, 2) for i in range(reference.tdim)),
-                    entity=(reference.tdim, 0))]
+            dofs = [PointEvaluation(
+                reference, tuple(sympy.Rational(1, 2) for i in range(reference.tdim)),
+                entity=(reference.tdim, 0))]
         else:
             points, _ = get_quadrature(variant, order + 1)
 
-            dofs = []
             for v_n, v in enumerate(reference.vertices):
                 dofs.append(PointEvaluation(reference, v, entity=(0, v_n)))
             for edim in range(1, 4):
-                for e_n, vs in enumerate(reference.sub_entities(edim)):
-                    entity = create_reference(
-                        reference.sub_entity_types[edim],
-                        vertices=tuple(reference.vertices[i] for i in vs))
+                for e_n in range(reference.sub_entity_count(edim)):
+                    entity = reference.sub_entity(edim, e_n)
                     for i in product(range(1, order), repeat=edim):
                         dofs.append(
                             PointEvaluation(
@@ -40,15 +48,15 @@ class Q(CiarletElement):
                                                  for j, o in enumerate(entity.origin)),
                                 entity=(edim, e_n)))
 
-        super().__init__(
-            reference, order,
-            quolynomial_set(reference.tdim, 1, order),
-            dofs,
-            reference.tdim,
-            1)
+        poly: typing.List[FunctionInput] = []
+        poly += quolynomial_set_1d(reference.tdim, order)
+
+        super().__init__(reference, order, poly, dofs, reference.tdim, 1)
         self.variant = variant
 
-    def get_tensor_factorisation(self):
+    def get_tensor_factorisation(
+        self
+    ) -> typing.List[typing.Tuple[str, typing.List[FiniteElement], typing.List[int]]]:
         """Get the representation of the element as a tensor product."""
         from symfem import create_element
         interval_q = create_element("interval", "Lagrange", self.order)
@@ -84,8 +92,12 @@ class Q(CiarletElement):
 
         return [("scalar", [interval_q for i in range(self.reference.tdim)], perm)]
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["Q", "Lagrange", "P"]
@@ -97,31 +109,42 @@ class Q(CiarletElement):
 class VectorQ(CiarletElement):
     """A vector Q element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
         scalar_space = Q(reference, order, variant)
-        dofs = []
+        dofs: ListOfFunctionals = []
+        poly: typing.List[FunctionInput] = []
         if reference.tdim == 1:
-            directions = [1]
+            for p in scalar_space.dofs:
+                dofs.append(PointEvaluation(reference, p.dof_point(), entity=p.entity))
+
+            poly += quolynomial_set_1d(reference.tdim, order)
         else:
             directions = [
                 tuple(1 if i == j else 0 for j in range(reference.tdim))
                 for i in range(reference.tdim)
             ]
-        for p in scalar_space.dofs:
-            for d in directions:
-                dofs.append(DotPointEvaluation(reference, p.point, d, entity=p.entity))
+            for p in scalar_space.dofs:
+                for d in directions:
+                    dofs.append(DotPointEvaluation(reference, p.dof_point(), d, entity=p.entity))
 
-        super().__init__(
-            reference, order,
-            quolynomial_set(reference.tdim, reference.tdim, order),
-            dofs,
-            reference.tdim,
-            reference.tdim,
-        )
+            poly += quolynomial_set_vector(reference.tdim, reference.tdim, order)
+
+        super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["vector Q", "vQ"]
@@ -133,11 +156,19 @@ class VectorQ(CiarletElement):
 class Nedelec(CiarletElement):
     """Nedelec Hcurl finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = quolynomial_set(reference.tdim, reference.tdim, order - 1)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += quolynomial_set_vector(reference.tdim, reference.tdim, order - 1)
         poly += Hcurl_quolynomials(reference.tdim, reference.tdim, order)
 
-        dofs = make_integral_moment_dofs(
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             edges=(TangentIntegralMoment, Q, order - 1, {"variant": variant}),
             faces=(IntegralMoment, RaviartThomas, order - 1, "covariant", {"variant": variant}),
@@ -147,8 +178,12 @@ class Nedelec(CiarletElement):
         super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["NCE", "RTCE", "Qcurl", "Nedelec", "Ncurl"]
@@ -160,11 +195,19 @@ class Nedelec(CiarletElement):
 class RaviartThomas(CiarletElement):
     """Raviart-Thomas Hdiv finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = quolynomial_set(reference.tdim, reference.tdim, order - 1)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += quolynomial_set_vector(reference.tdim, reference.tdim, order - 1)
         poly += Hdiv_quolynomials(reference.tdim, reference.tdim, order)
 
-        dofs = make_integral_moment_dofs(
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             facets=(NormalIntegralMoment, Q, order - 1, {"variant": variant}),
             cells=(IntegralMoment, Nedelec, order - 1, "contravariant", {"variant": variant}),
@@ -173,8 +216,12 @@ class RaviartThomas(CiarletElement):
         super().__init__(reference, order, poly, dofs, reference.tdim, reference.tdim)
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["NCF", "RTCF", "Qdiv"]
