@@ -4,33 +4,47 @@ This element's definition appears in https://doi.org/10.2307/2006147
 (Ciarlet, 1978)
 """
 
+import typing
+
 import sympy
+
 from ..finite_element import CiarletElement
-from ..functionals import PointEvaluation, DerivativePointEvaluation
-from ..polynomials import polynomial_set
-from ..symbolic import PiecewiseFunction, x
+from ..functionals import DerivativePointEvaluation, ListOfFunctionals, PointEvaluation
+from ..functions import FunctionInput
+from ..piecewise_functions import PiecewiseFunction
+from ..references import Reference
+from ..symbols import x
 
 
 class P1Hermite(CiarletElement):
     """P1Hermite finite element."""
 
-    def __init__(self, reference, order, poly):
+    def __init__(self, reference: Reference, order: int, poly: typing.List[FunctionInput]):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            poly: The polynomial basis
+        """
         assert order == 3
-        dofs = []
+        dofs: ListOfFunctionals = []
         for v_n, vs in enumerate(reference.vertices):
             dofs.append(PointEvaluation(reference, vs, entity=(0, v_n)))
             dofs.append(DerivativePointEvaluation(reference, vs, (1, 0), entity=(0, v_n)))
             dofs.append(DerivativePointEvaluation(reference, vs, (0, 1), entity=(0, v_n)))
 
-        super().__init__(
-            reference, order, poly, dofs, reference.tdim, 1
-        )
+        super().__init__(reference, order, poly, dofs, reference.tdim, 1)
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"poly": self._basis}
 
-    names = []
+    names: typing.List[str] = []
     references = ["triangle"]
     min_order = 3
     max_order = 3
@@ -40,11 +54,17 @@ class P1Hermite(CiarletElement):
 class ReducedHsiehCloughTocher(CiarletElement):
     """Reduced Hsieh-Clough-Tocher finite element."""
 
-    def __init__(self, reference, order):
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+        """
         from symfem import create_reference
         assert order == 3
         assert reference.name == "triangle"
-        dofs = []
+        dofs: ListOfFunctionals = []
         for v_n, vs in enumerate(reference.vertices):
             dofs.append(PointEvaluation(reference, vs, entity=(0, v_n)))
             dofs.append(DerivativePointEvaluation(reference, vs, (1, 0), entity=(0, v_n)))
@@ -59,21 +79,27 @@ class ReducedHsiehCloughTocher(CiarletElement):
 
         refs = [create_reference("triangle", vs) for vs in subs]
 
-        polys = [
-            polynomial_set(reference.tdim, 1, order),
-            [],
-            polynomial_set(reference.tdim, 1, order),
-        ]
-        polys[0].remove(x[0] ** 2 * x[1])
-        polys[1] = [1, x[0], x[0] ** 2, x[1], x[0] * x[1], x[1] ** 2,
-                    x[0] * x[1] ** 2 - x[0] ** 2 * x[1],
-                    x[0] ** 3 - x[1] ** 3, x[0] ** 3 + 3 * x[0] * x[1] ** 2]
-        polys[2].remove(x[0] * x[1] ** 2)
+        polys: typing.List[typing.List[FunctionInput]] = [[], [], []]
+        for i in range(order + 1):
+            for j in range(order + 1 - i):
+                if i != 2 or j != 1:
+                    polys[0].append(x[0] ** i * x[1] ** j)
+        polys[1] += [1, x[0], x[0] ** 2, x[1], x[0] * x[1], x[1] ** 2,
+                     x[0] * x[1] ** 2 - x[0] ** 2 * x[1],
+                     x[0] ** 3 - x[1] ** 3, x[0] ** 3 + 3 * x[0] * x[1] ** 2]
+        for i in range(order + 1):
+            for j in range(order + 1 - i):
+                if i != 1 or j != 2:
+                    polys[2].append(x[0] ** i * x[1] ** j)
 
-        bases = [P1Hermite(r, 3, p).get_basis_functions()
-                 for r, p in zip(refs, polys)]
+        bases = []
+        for r, p in zip(refs, polys):
+            bf = []
+            for f in P1Hermite(r, 3, p).get_basis_functions():
+                bf.append(f)
+            bases.append(bf)
 
-        piece_list = []
+        piece_list: typing.List[typing.Tuple[FunctionInput, ...]] = []
         piece_list.append((bases[0][0], 0, bases[2][3]))
         piece_list.append((bases[0][1], 0, bases[2][4]))
         piece_list.append((bases[0][2], 0, bases[2][5]))
@@ -88,8 +114,9 @@ class ReducedHsiehCloughTocher(CiarletElement):
         piece_list.append((0, bases[1][4], bases[2][1]))
         piece_list.append((0, bases[1][5], bases[2][2]))
 
-        poly = [
-            PiecewiseFunction(list(zip(subs, p)))
+        poly: typing.List[FunctionInput] = []
+        poly += [
+            PiecewiseFunction({i: j for i, j in zip(subs, p)}, 2)
             for p in piece_list
         ]
 
