@@ -1,29 +1,38 @@
 """Taylor element on an interval, triangle or tetrahedron."""
 
-from ..finite_element import CiarletElement
-from ..moments import make_integral_moment_dofs
-from ..polynomials import polynomial_set
-from ..functionals import IntegralMoment, DerivativePointEvaluation
-from ..symbolic import sym_sum
-from .lagrange import Lagrange
+import typing
 from itertools import product
 
+from ..finite_element import CiarletElement
+from ..functionals import DerivativePointEvaluation, IntegralMoment, ListOfFunctionals
+from ..functions import FunctionInput
+from ..moments import make_integral_moment_dofs
+from ..polynomials import polynomial_set_1d
+from ..references import Reference
+from .lagrange import Lagrange
+
 
 class Taylor(CiarletElement):
     """Taylor finite element."""
 
-    def __init__(self, reference, order):
-        dofs = make_integral_moment_dofs(
+    def __init__(self, reference: Reference, order: int):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+        """
+        dofs: ListOfFunctionals = make_integral_moment_dofs(
             reference,
             cells=(IntegralMoment, Lagrange, 0, {"variant": "equispaced"}),
         )
-        midpoint = tuple(sym_sum(i) / len(i) for i in zip(*reference.vertices))
         for i in product(range(order + 1), repeat=reference.tdim):
             if 1 <= sum(i) <= order:
                 dofs.append(DerivativePointEvaluation(
-                    reference, midpoint, i, entity=(reference.tdim, 0)))
+                    reference, reference.midpoint(), i, entity=(reference.tdim, 0)))
 
-        poly = polynomial_set(reference.tdim, 1, order)
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_1d(reference.tdim, order)
 
         super().__init__(reference, order, poly, dofs, reference.tdim, 1)
 
