@@ -4,41 +4,51 @@ These elements' definitions appear in https://doi.org/10.1090/S0025-5718-2013-02
 (Cockburn, Qiu, 2013)
 """
 
+import typing
 from itertools import product
+
+import sympy
+
 from ..finite_element import CiarletElement
-from ..polynomials import quolynomial_set, orthogonal_basis
-from ..functionals import (TangentIntegralMoment, IntegralAgainst,
-                           NormalIntegralMoment, PointEvaluation,
-                           DerivativeIntegralMoment)
-from ..symbolic import x, t, subs
-from ..calculus import grad, curl
+from ..functionals import (DerivativeIntegralMoment, IntegralAgainst, ListOfFunctionals,
+                           NormalIntegralMoment, PointEvaluation, TangentIntegralMoment)
+from ..functions import FunctionInput, ScalarFunction, VectorFunction
 from ..moments import make_integral_moment_dofs
-from ..vectors import vcross
+from ..polynomials import orthogonal_basis, quolynomial_set_1d, quolynomial_set_vector
+from ..references import Reference
+from ..symbols import t, x
 from .q import Q
 
 
-def p(k, v):
+def p(k: int, v: sympy.core.symbol.Symbol) -> ScalarFunction:
     """Return the kth Legendre polynomial."""
     return orthogonal_basis("interval", k, 0, [v])[0][-1]
 
 
-def b(k, v):
-    """
-    Return the function B_k.
+def b(k: int, v: sympy.core.symbol.Symbol) -> ScalarFunction:
+    """Return the function B_k.
 
     This function is defined on page 4 (606) of
     https://doi.org/10.1090/S0025-5718-2013-02729-9 (Cockburn, Qiu, 2013).
     """
     if k == 1:
-        return 0
+        return ScalarFunction(0)
     return (p(k, v) - p(k - 2, v)) / (4 + k - 2)
 
 
 class TNT(CiarletElement):
     """TiNiest Tensor scalar finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = quolynomial_set(reference.tdim, 1, order)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += quolynomial_set_1d(reference.tdim, order)
         if reference.tdim == 2:
             for i in range(2):
                 variables = [x[j] for j in range(2) if j != i]
@@ -52,7 +62,7 @@ class TNT(CiarletElement):
                     for f1 in [1 - variables[1], variables[1]]:
                         poly.append(f0 * f1 * b(order + 1, x[i]))
 
-        dofs = []
+        dofs: ListOfFunctionals = []
         for i, v in enumerate(reference.vertices):
             dofs.append(PointEvaluation(reference, v, entity=(0, i)))
 
@@ -73,21 +83,26 @@ class TNT(CiarletElement):
                         reference, face, delta_f, entity=(2, face_n), mapping="identity"))
 
         if reference.tdim == 3:
-            for i in product(range(1, order), repeat=3):
-                f = 1
-                for j, k in zip(i, x):
+            dummy_dof = PointEvaluation(reference, reference.midpoint(), (3, 0))
+            for ii in product(range(1, order), repeat=3):
+                f = sympy.Integer(1)
+                for j, k in zip(ii, x):
                     f *= k ** j * (k - 1)
-                grad_f = tuple(j.expand() for j in grad(f, 3))
+                grad_f = tuple(sympy.S(j).expand() for j in ScalarFunction(f).grad(3))
                 dofs.append(DerivativeIntegralMoment(
-                    reference, reference, 1, grad_f, None, entity=(3, 0), mapping="identity"))
+                    reference, reference, 1, grad_f, dummy_dof, entity=(3, 0), mapping="identity"))
 
         super().__init__(
             reference, order, poly, dofs, reference.tdim, 1
         )
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["tiniest tensor", "TNT"]
@@ -99,34 +114,40 @@ class TNT(CiarletElement):
 class TNTcurl(CiarletElement):
     """TiNiest Tensor Hcurl finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = quolynomial_set(reference.tdim, reference.tdim, order)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += quolynomial_set_vector(reference.tdim, reference.tdim, order)
         if reference.tdim == 2:
-            for i in product([0, 1], repeat=2):
-                if sum(i) != 0:
-                    poly.append([j.expand() for j in [
-                        p(order, i[0] * x[0]) * b(order + 1, i[1] * x[1]),
-                        -b(order + 1, i[0] * x[0]) * p(order, i[1] * x[1]),
-                    ]])
+            for ii in product([0, 1], repeat=2):
+                if sum(ii) != 0:
+                    poly.append(tuple(sympy.S(j).expand() for j in [
+                        p(order, ii[0] * x[0]) * b(order + 1, ii[1] * x[1]),
+                        -b(order + 1, ii[0] * x[0]) * p(order, ii[1] * x[1])]))
         else:
             face_poly = []
-            for i in product([0, 1], repeat=2):
-                if sum(i) != 0:
-                    face_poly.append([j.expand() for j in [
-                        b(order + 1, i[0] * t[0]) * p(order, i[1] * t[1]),
-                        p(order, i[0] * t[0]) * b(order + 1, i[1] * t[1]),
-                    ]])
+            for ii in product([0, 1], repeat=2):
+                if sum(ii) != 0:
+                    face_poly.append(tuple(sympy.S(j).expand() for j in [
+                        b(order + 1, ii[0] * t[0]) * p(order, ii[1] * t[1]),
+                        p(order, ii[0] * t[0]) * b(order + 1, ii[1] * t[1])]))
             for lamb_n in [(x[0], 0, 0), (1 - x[0], 0, 0),
                            (0, x[1], 0), (0, 1 - x[1], 0),
                            (0, 0, x[2]), (0, 0, 1 - x[2])]:
                 variables = tuple(i for i, j in enumerate(lamb_n) if j == 0)
-                poly += [vcross(lamb_n, [
-                    subs(p[0], t[:2], [x[j] for j in variables]) if i == variables[0]
-                    else (subs(p[1], t[:2], [x[j] for j in variables]) if i == variables[1] else 0)
-                    for i in range(3)
-                ]) for p in face_poly]
+                for pf in face_poly:
+                    psub = VectorFunction(pf).subs(t[:2], [x[j] for j in variables])
+                    pc = VectorFunction(lamb_n).cross(VectorFunction([
+                        psub[variables.index(i)] if i in variables else 0 for i in range(3)]))
+                    poly.append(pc)
 
-        dofs = []
+        dofs: ListOfFunctionals = []
         dofs += make_integral_moment_dofs(
             reference,
             edges=(TangentIntegralMoment, Q, order, {"variant": variant}),
@@ -134,18 +155,18 @@ class TNTcurl(CiarletElement):
 
         # Face moments
         face_moments = []
-        for i in product(range(order + 1), repeat=2):
-            if sum(i) > 0:
-                f = x[0] ** i[0] * x[1] ** i[1]
-                grad_f = grad(f, 2)
-                grad_f = subs((grad_f[1], -grad_f[0]), x[:2], t[:2])
-                face_moments.append(grad_f)
+        for ii in product(range(order + 1), repeat=2):
+            if sum(ii) > 0:
+                f = x[0] ** ii[0] * x[1] ** ii[1]
+                grad_f = ScalarFunction(f).grad(2)
+                grad_f2 = VectorFunction((grad_f[1], -grad_f[0])).subs(x[:2], tuple(t[:2]))
+                face_moments.append(grad_f2)
 
         for i in range(2, order + 1):
             for j in range(2, order + 1):
-                face_moments.append((
+                face_moments.append(VectorFunction((
                     t[1] ** (j - 1) * (1 - t[1]) * t[0] ** (i - 2) * (i * t[0] - i + 1),
-                    -t[0] ** (i - 1) * (1 - t[0]) * t[1] ** (j - 2) * (j - 1 - j * t[1])))
+                    -t[0] ** (i - 1) * (1 - t[0]) * t[1] ** (j - 2) * (j - 1 - j * t[1]))))
         if reference.tdim == 2:
             for f in face_moments:
                 dofs.append(IntegralAgainst(
@@ -164,19 +185,19 @@ class TNTcurl(CiarletElement):
                     for k in range(order + 1):
                         f = (x[0] ** k * x[1] ** i * (1 - x[1]) * x[2] ** j * (1 - x[2]), 0, 0)
                         dofs.append(IntegralAgainst(
-                            reference, reference, curl(curl(f)), entity=(3, 0),
+                            reference, reference, VectorFunction(f).curl().curl(), entity=(3, 0),
                             mapping="covariant"))
 
-                        f = (0, x[1] ** k * x[0] ** i * (1 - x[0]) * x[2] ** j * (1 - x[2]), 0, 0)
+                        f = (0, x[1] ** k * x[0] ** i * (1 - x[0]) * x[2] ** j * (1 - x[2]), 0)
                         dofs.append(IntegralAgainst(
-                            reference, reference, curl(curl(f)), entity=(3, 0),
+                            reference, reference, VectorFunction(f).curl().curl(), entity=(3, 0),
                             mapping="covariant"))
 
                         if k in [0, 2]:
                             f = (0, 0,  x[2] ** k * x[0] ** i * (1 - x[0]) * x[1] ** j * (1 - x[1]))
                             dofs.append(IntegralAgainst(
-                                reference, reference, curl(curl(f)), entity=(3, 0),
-                                mapping="covariant"))
+                                reference, reference, VectorFunction(f).curl().curl(),
+                                entity=(3, 0), mapping="covariant"))
 
             for i in range(2, order + 1):
                 for j in range(2, order + 1):
@@ -184,7 +205,7 @@ class TNTcurl(CiarletElement):
                         f = x[0] ** (i - 1) * x[0] ** i
                         f *= x[1] ** (j - 1) * x[1] ** j
                         f *= x[2] ** (k - 1) * x[2] ** k
-                        grad_f = grad(f, 3)
+                        grad_f = ScalarFunction(f).grad(3)
                         dofs.append(IntegralAgainst(
                             reference, reference, grad_f, entity=(3, 0), mapping="contravariant"))
 
@@ -193,8 +214,12 @@ class TNTcurl(CiarletElement):
         )
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["tiniest tensor Hcurl", "TNTcurl"]
@@ -206,37 +231,48 @@ class TNTcurl(CiarletElement):
 class TNTdiv(CiarletElement):
     """TiNiest Tensor Hdiv finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = quolynomial_set(reference.tdim, reference.tdim, order)
-        if reference.tdim == 2:
-            for i in product([0, 1], repeat=2):
-                if sum(i) != 0:
-                    poly.append([j.expand() for j in [
-                        b(order + 1, i[0] * x[0]) * p(order, i[1] * x[1]),
-                        p(order, i[0] * x[0]) * b(order + 1, i[1] * x[1]),
-                    ]])
-        else:
-            for i in product([0, 1], repeat=3):
-                if sum(i) != 0:
-                    poly.append([
-                        b(order + 1, i[0] * x[0]) * p(order, i[1] * x[1]) * p(order, i[2] * x[2]),
-                        p(order, i[0] * x[0]) * b(order + 1, i[1] * x[1]) * p(order, i[2] * x[2]),
-                        p(order, i[0] * x[0]) * p(order, i[1] * x[1]) * b(order + 1, i[2] * x[2]),
-                    ])
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
 
-        dofs = []
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += quolynomial_set_vector(reference.tdim, reference.tdim, order)
+        if reference.tdim == 2:
+            for ii in product([0, 1], repeat=2):
+                if sum(ii) != 0:
+                    poly.append(tuple(sympy.S(j).expand() for j in [
+                        b(order + 1, ii[0] * x[0]) * p(order, ii[1] * x[1]),
+                        p(order, ii[0] * x[0]) * b(order + 1, ii[1] * x[1]),
+                    ]))
+        else:
+            for ii in product([0, 1], repeat=3):
+                if sum(ii) != 0:
+                    poly.append((
+                        b(order + 1,
+                          ii[0] * x[0]) * p(order, ii[1] * x[1]) * p(order, ii[2] * x[2]),
+                        p(order,
+                          ii[0] * x[0]) * b(order + 1, ii[1] * x[1]) * p(order, ii[2] * x[2]),
+                        p(order,
+                          ii[0] * x[0]) * p(order, ii[1] * x[1]) * b(order + 1, ii[2] * x[2]),
+                    ))
+
+        dofs: ListOfFunctionals = []
         dofs += make_integral_moment_dofs(
             reference,
             facets=(NormalIntegralMoment, Q, order, {"variant": variant}),
         )
 
-        for i in product(range(order + 1), repeat=reference.tdim):
-            if sum(i) > 0:
+        for ii in product(range(order + 1), repeat=reference.tdim):
+            if sum(ii) > 0:
                 if reference.tdim == 2:
-                    f = x[0] ** i[0] * x[1] ** i[1]
+                    f = x[0] ** ii[0] * x[1] ** ii[1]
                 else:
-                    f = x[0] ** i[0] * x[1] ** i[1] * x[2] ** i[2]
-                grad_f = grad(f, reference.tdim)
+                    f = x[0] ** ii[0] * x[1] ** ii[1] * x[2] ** ii[2]
+                grad_f = ScalarFunction(f).grad(reference.tdim)
                 dofs.append(IntegralAgainst(
                     reference, reference, grad_f, entity=(reference.tdim, 0), mapping="covariant"))
 
@@ -288,8 +324,12 @@ class TNTdiv(CiarletElement):
         )
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["tiniest tensor Hdiv", "TNTdiv"]
