@@ -5,22 +5,32 @@ These elements' definitions appear in https://doi.org/10.1137/16M1073352
 (Gilette, Kloefkorn, 2018)
 """
 
+import typing
+
 from ..finite_element import CiarletElement
-from ..polynomials import polynomial_set
-from ..functionals import (IntegralMoment, TangentIntegralMoment, IntegralAgainst,
-                           NormalIntegralMoment)
-from ..symbolic import x, t, subs
-from ..calculus import grad, curl
-from ..vectors import vcross
+from ..functionals import (IntegralAgainst, IntegralMoment, ListOfFunctionals, NormalIntegralMoment,
+                           TangentIntegralMoment)
+from ..functions import FunctionInput, ScalarFunction, VectorFunction
 from ..moments import make_integral_moment_dofs
+from ..polynomials import polynomial_set_vector
+from ..references import Reference
+from ..symbols import t, x
 from .dpc import DPC, VectorDPC
 
 
 class TrimmedSerendipityHcurl(CiarletElement):
     """Trimmed serendipity Hcurl finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = polynomial_set(reference.tdim, reference.tdim, order - 1)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_vector(reference.tdim, reference.tdim, order - 1)
         if reference.tdim == 2:
             poly += [
                 (x[0] ** j * x[1] ** (order - j), -x[0] ** (j + 1) * x[1] ** (order - 1 - j))
@@ -31,23 +41,29 @@ class TrimmedSerendipityHcurl(CiarletElement):
                 poly += [(x[1] ** order, order * x[0] * x[1] ** (order - 1)),
                          (order * x[0] ** (order - 1) * x[1], x[0] ** order)]
         else:
-            poly += [
-                vcross(x, [x[0] ** i * x[1] ** j * x[2] ** (order - 1 - i - j)
-                           if d == dim else 0 for d in range(3)])
-                for i in range(order) for j in range(order - i) for dim in range(3)
-                if i == 0 or dim != 0]
+            for i in range(order):
+                for j in range(order - i):
+                    for dim in range(3):
+                        if i == 0 or dim != 0:
+                            p = VectorFunction(tuple(x)).cross(VectorFunction([
+                                x[0] ** i * x[1] ** j * x[2] ** (order - 1 - i - j)
+                                if d == dim else 0 for d in range(3)]))
+                            poly.append(p)
 
             if order == 1:
-                poly += [grad(x[0] * x[1] * x[2] ** order, 3)]
+                poly += [ScalarFunction(x[0] * x[1] * x[2] ** order).grad(3)]
             else:
-                poly += [grad(x[0] * x[1] * x[2] ** order, 3),
-                         grad(x[0] * x[1] ** order * x[2], 3),
-                         grad(x[0] ** order * x[1] * x[2], 3)]
-            poly += [grad(x[0] * x[1] ** i * x[2] ** (order - i), 3) for i in range(order + 1)]
-            poly += [grad(x[1] * x[0] ** i * x[2] ** (order - i), 3) for i in range(order + 1)
-                     if i != 1]
-            poly += [grad(x[2] * x[0] ** i * x[1] ** (order - i), 3) for i in range(order + 1)
-                     if i != 1 and order - i != 1]
+                poly += [ScalarFunction(x[0] * x[1] * x[2] ** order).grad(3),
+                         ScalarFunction(x[0] * x[1] ** order * x[2]).grad(3),
+                         ScalarFunction(x[0] ** order * x[1] * x[2]).grad(3)]
+            for i in range(order + 1):
+                poly.append(ScalarFunction(x[0] * x[1] ** i * x[2] ** (order - i)).grad(3))
+                if i != 1:
+                    poly.append(
+                        ScalarFunction(x[1] * x[0] ** i * x[2] ** (order - i)).grad(3))
+                    if order - i != 1:
+                        poly.append(
+                            ScalarFunction(x[2] * x[0] ** i * x[1] ** (order - i)).grad(3))
             for i in range(order):
                 p = x[0] * x[1] ** i * x[2] ** (order - 1 - i)
                 poly.append((0, -x[2] * p, x[1] * p))
@@ -57,7 +73,7 @@ class TrimmedSerendipityHcurl(CiarletElement):
                     p = x[2] * x[0] ** i * x[1] ** (order - 1 - i)
                     poly.append((-x[1] * p, x[0] * p, 0))
 
-        dofs = []
+        dofs: ListOfFunctionals = []
         dofs += make_integral_moment_dofs(
             reference,
             edges=(TangentIntegralMoment, DPC, order - 1, {"variant": variant}),
@@ -70,18 +86,22 @@ class TrimmedSerendipityHcurl(CiarletElement):
             for f_n in range(reference.sub_entity_count(2)):
                 face = reference.sub_entity(2, f_n)
                 for i in range(order):
-                    f = grad(x[0] ** (order - 1 - i) * x[1] ** i, 2)
-                    f = subs([f[1], -f[0]], x, t)
+                    f = ScalarFunction(x[0] ** (order - 1 - i) * x[1] ** i).grad(2)
+                    f2 = VectorFunction((f[1], -f[0])).subs(x, tuple(t))
                     dofs.append(IntegralAgainst(
-                        reference, face, f, entity=(2, f_n), mapping="contravariant"))
+                        reference, face, f2, entity=(2, f_n), mapping="contravariant"))
 
         super().__init__(
             reference, order, poly, dofs, reference.tdim, reference.tdim
         )
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["trimmed serendipity Hcurl", "TScurl"]
@@ -93,8 +113,16 @@ class TrimmedSerendipityHcurl(CiarletElement):
 class TrimmedSerendipityHdiv(CiarletElement):
     """Trimmed serendipity Hdiv finite element."""
 
-    def __init__(self, reference, order, variant="equispaced"):
-        poly = polynomial_set(reference.tdim, reference.tdim, order - 1)
+    def __init__(self, reference: Reference, order: int, variant: str = "equispaced"):
+        """Create the element.
+
+        Args:
+            reference: The reference element
+            order: The polynomial order
+            variant: The variant of the element
+        """
+        poly: typing.List[FunctionInput] = []
+        poly += polynomial_set_vector(reference.tdim, reference.tdim, order - 1)
         if reference.tdim == 2:
             poly += [
                 (x[0] ** (j + 1) * x[1] ** (order - 1 - j), x[0] ** j * x[1] ** (order - j))
@@ -111,14 +139,17 @@ class TrimmedSerendipityHdiv(CiarletElement):
                     poly.append((x[0] * p, x[1] * p, x[2] * p))
             for i in range(order):
                 p = x[0] * x[1] ** i * x[2] ** (order - 1 - i)
-                poly.append(curl((0, -x[2] * p, x[1] * p)))
+                poly.append(
+                    VectorFunction((0, -x[2] * p, x[1] * p)).curl())
                 p = x[1] * x[0] ** i * x[2] ** (order - 1 - i)
-                poly.append(curl((-x[2] * p, 0, x[0] * p)))
+                poly.append(
+                    VectorFunction((-x[2] * p, 0, x[0] * p)).curl())
                 if order > 1:
                     p = x[2] * x[0] ** i * x[1] ** (order - 1 - i)
-                    poly.append(curl((-x[1] * p, x[0] * p, 0)))
+                    poly.append(
+                        VectorFunction((-x[1] * p, x[0] * p, 0)).curl())
 
-        dofs = []
+        dofs: ListOfFunctionals = []
         dofs += make_integral_moment_dofs(
             reference,
             facets=(NormalIntegralMoment, DPC, order - 1, {"variant": variant}),
@@ -127,14 +158,14 @@ class TrimmedSerendipityHdiv(CiarletElement):
         )
         if order >= 2:
             if reference.tdim == 2:
-                fs = [grad(x[0] ** (order - 1 - i) * x[1] ** i, 2)
+                fs = [ScalarFunction(x[0] ** (order - 1 - i) * x[1] ** i).grad(2)
                       for i in range(order)]
             else:
-                fs = [grad(x[0] ** (order - 1 - i - j) * x[1] ** i * x[2] ** j, 3)
+                fs = [ScalarFunction(x[0] ** (order - 1 - i - j) * x[1] ** i * x[2] ** j).grad(3)
                       for i in range(order) for j in range(order - i)]
             for f in fs:
-                f = subs(f, x, t)
-                dofs.append(IntegralAgainst(reference, reference, f, entity=(reference.tdim, 0),
+                f2 = f.subs(x, tuple(t))
+                dofs.append(IntegralAgainst(reference, reference, f2, entity=(reference.tdim, 0),
                                             mapping="covariant"))
 
         super().__init__(
@@ -142,8 +173,12 @@ class TrimmedSerendipityHdiv(CiarletElement):
         )
         self.variant = variant
 
-    def init_kwargs(self):
-        """Return the kwargs used to create this element."""
+    def init_kwargs(self) -> typing.Dict[str, typing.Any]:
+        """Return the kwargs used to create this element.
+
+        Returns:
+            Keyword argument dictionary
+        """
         return {"variant": self.variant}
 
     names = ["trimmed serendipity Hdiv", "TSdiv"]
