@@ -4,6 +4,7 @@ import pytest
 
 
 def pytest_addoption(parser):
+    """Add parser options."""
     parser.addoption("--elements-to-test", action="store", default="ALL")
     parser.addoption("--cells-to-test", action="store", default="ALL")
     parser.addoption("--has-basix", action="store", default="0")
@@ -12,6 +13,7 @@ def pytest_addoption(parser):
 
 @pytest.fixture
 def elements_to_test(request):
+    """Get list of elements to include in tests."""
     data = request.config.getoption("--elements-to-test")
     if data == "ALL":
         return "ALL"
@@ -20,11 +22,13 @@ def elements_to_test(request):
 
 @pytest.fixture
 def speed(request):
+    """Get test speed."""
     return request.config.getoption("--speed")
 
 
 @pytest.fixture
 def cells_to_test(request):
+    """Get list of cells to include in tests."""
     data = request.config.getoption("--cells-to-test")
     if data == "ALL":
         return "ALL"
@@ -33,5 +37,6 @@ def cells_to_test(request):
 
 @pytest.fixture
 def has_basix(request):
+    """Get has-basix flag."""
     data = request.config.getoption("--has-basix")
     return data == "1"
