@@ -1,8 +1,14 @@
-from symfem import create_element
-import numpy as np
+"""Test that basis functions agree with Basix."""
+
+import typing
+
 import pytest
 
-elements = {
+from symfem import create_element
+
+elements: typing.Dict[str, typing.List[typing.Tuple[
+    str, str, typing.Iterable, typing.List[typing.Tuple[str, typing.Any]]
+]]] = {
     "interval": [
         ("P", "Lagrange", range(1, 4), [("LagrangeVariant", "equispaced")]),
         ("serendipity", "Serendipity", range(1, 5), [("LagrangeVariant", "equispaced"),
@@ -63,7 +69,16 @@ def to_float(a):
         return [to_float(i) for i in a]
 
 
+def to_nparray(a):
+    import numpy as np
+    try:
+        return float(a)
+    except:  # noqa: E722
+        return np.array([to_float(i) for i in a])
+
+
 def make_lattice(cell, N=3):
+    import numpy as np
     if cell == "interval":
         return np.array([[i / N] for i in range(N + 1)])
     if cell == "triangle":
@@ -113,6 +128,8 @@ def test_against_basix(has_basix, elements_to_test, cells_to_test, cell, symfem_
         except ImportError:
             pytest.skip("Basix must be installed to run this test.")
 
+    import numpy as np
+
     points = make_lattice(cell, 2)
     parsed_args = []
     for a in args:
@@ -130,7 +147,7 @@ def test_against_basix(has_basix, elements_to_test, cells_to_test, cell, symfem_
     result = space.tabulate(0, points)[0]
 
     element = create_element(cell, symfem_type, order)
-    sym_result = element.tabulate_basis(points, "xyz,xyz", symbolic=False)
+    sym_result = to_nparray(element.tabulate_basis(points, "xyz,xyz"))
 
     if len(result.shape) != len(sym_result.shape):
         sym_result = sym_result.reshape(result.shape)
