@@ -1,6 +1,9 @@
-import symfem
+"""Test Bernstein elements."""
+
 import pytest
 
+import symfem
+
 
 @pytest.mark.parametrize("celltype, degree", [(c, i) for c, n in [
     ("interval", 4), ("triangle", 3), ("tetrahedron", 2)
@@ -11,7 +14,7 @@ def test_bernstein(celltype, degree):
 
     for f in b.get_basis_functions():
         for p in poly:
-            if (f - p).simplify() == 0:
+            if f == p:
                 poly.remove(p)
                 break
         else:
