@@ -1,11 +1,15 @@
+"""Test polynomials."""
+
+from random import choice
+
 import pytest
 import sympy
-from symfem.polynomials import Hdiv_polynomials, Hcurl_polynomials, orthogonal_basis
-from symfem.vectors import vdot
-from symfem.symbolic import x, t, subs
-from symfem.calculus import div
-from symfem import create_reference, create_element
-from random import choice
+
+from symfem import create_element, create_reference
+from symfem.functions import VectorFunction
+from symfem.polynomials import (Hcurl_polynomials, Hdiv_polynomials, orthogonal_basis,
+                                orthonormal_basis)
+from symfem.symbols import t, x
 
 
 @pytest.mark.parametrize("reference", ["triangle", "tetrahedron"])
@@ -24,7 +28,7 @@ def test_Hcurl_space(reference, order):
     ref = create_reference(reference)
     polynomials = Hcurl_polynomials(ref.tdim, ref.tdim, order)
     for p in polynomials:
-        assert vdot(p, x) == 0
+        assert p.dot(VectorFunction(x[:ref.tdim])) == 0
 
 
 @pytest.mark.parametrize("reference", ["triangle"])
@@ -32,13 +36,13 @@ def test_MTW_space(reference):
     e = create_element(reference, "MTW", 3)
     polynomials = e.get_polynomial_basis()
     for p in polynomials:
-        assert div(p).is_real
+        assert p.div().as_sympy().is_real
         for vs in e.reference.sub_entities(1):
             sub_ref = create_reference(e.reference.sub_entity_types[1],
                                        [e.reference.vertices[i] for i in vs])
-            p_edge = subs(p, x, [i + t[0] * j
-                                 for i, j in zip(sub_ref.origin, sub_ref.axes[0])])
-            poly = vdot(p_edge, sub_ref.normal()).expand().simplify()
+            p_edge = p.subs(x, [i + t[0] * j
+                                for i, j in zip(sub_ref.origin, sub_ref.axes[0])])
+            poly = p_edge.dot(sub_ref.normal()).as_sympy().expand().simplify()
             assert poly.is_real or sympy.Poly(poly).degree() <= 1
 
 
@@ -54,13 +58,13 @@ def test_BDFM_space(reference, order):
             sub_ref = create_reference(e.reference.sub_entity_types[tdim - 1],
                                        [e.reference.vertices[i] for i in vs])
             if tdim == 2:
-                p_edge = subs(p, x, [i + t[0] * j
-                                     for i, j in zip(sub_ref.origin, sub_ref.axes[0])])
+                p_edge = p.subs(x, [i + t[0] * j
+                                    for i, j in zip(sub_ref.origin, sub_ref.axes[0])])
             else:
-                p_edge = subs(p, x, [i + t[0] * j + t[1] * k
-                                     for i, j, k in zip(sub_ref.origin, sub_ref.axes[0],
-                                                        sub_ref.axes[1])])
-            poly = vdot(p_edge, sub_ref.normal()).expand().simplify()
+                p_edge = p.subs(x, [i + t[0] * j + t[1] * k
+                                    for i, j, k in zip(sub_ref.origin, sub_ref.axes[0],
+                                                       sub_ref.axes[1])])
+            poly = p_edge.dot(sub_ref.normal()).as_sympy().expand().simplify()
             assert poly.is_real or sympy.Poly(poly).degree() <= order - 1
 
 
@@ -69,19 +73,22 @@ def test_BDFM_space(reference, order):
     "tetrahedron", "hexahedron", "prism",
     "pyramid"])
 @pytest.mark.parametrize("order", range(5))
-def test_orthogonal_polynomials(reference, order):
+def test_orthogonal_polynomials(reference, order, speed):
+    if speed == "fast" and order > 2:
+        pytest.skip()
+
     polynomials = orthogonal_basis(reference, order, 0)[0]
     ref = create_reference(reference)
     if len(polynomials) <= 5:
         for i, p in enumerate(polynomials):
             for q in polynomials[:i]:
-                assert ref.integral(p * q, x) == 0
+                assert (p * q).integral(ref, x) == 0
     else:
         for _ in range(15):
             p = choice(polynomials)
             q = choice(polynomials)
             if p != q:
-                assert ref.integral(p * q, x) == 0
+                assert (p * q).integral(ref, x) == 0
 
 
 @pytest.mark.parametrize("reference", [
@@ -110,7 +117,22 @@ def test_orthogonal_polynomial_derivatives(reference, order):
 
     for i, j in first_d:
         for p, q in zip(polynomials[0], polynomials[i]):
-            assert (p.diff(x[j]) - q).simplify() == 0
+            assert p.diff(x[j]) == q
     for i, j, k in second_d:
         for p, q in zip(polynomials[0], polynomials[i]):
-            assert (p.diff(x[j]).diff(x[k]) - q).simplify() == 0
+            assert p.diff(x[j]).diff(x[k]) == q
+
+
+@pytest.mark.parametrize("reference", [
+    "interval", "triangle", "quadrilateral",
+    "tetrahedron", "hexahedron", "prism",
+    "pyramid"])
+@pytest.mark.parametrize("order", range(3))
+def test_orthonormal_polynomials(reference, order, speed):
+    if speed == "fast" and order > 2:
+        pytest.skip()
+
+    polynomials = orthonormal_basis(reference, order, 0)[0]
+    ref = create_reference(reference)
+    for p in polynomials:
+        assert (p ** 2).integral(ref, x) == 1
