@@ -1,7 +1,11 @@
+"""Test quadrature rules."""
+
 import pytest
 import sympy
+
 from symfem import quadrature
-from symfem.symbolic import symequal
+from symfem.functions import ScalarFunction
+from symfem.utils import allequal
 
 
 @pytest.mark.parametrize("order", range(1, 7))
@@ -9,9 +13,9 @@ def test_equispaced(order):
     points, weights = quadrature.equispaced(order + 1)
 
     x = sympy.Symbol("x")
-    poly = x
+    poly = ScalarFunction(x)
 
-    assert symequal(
+    assert allequal(
         poly.integrate((x, 0, 1)),
         sum(i * poly.subs(x, j) for i, j in zip(weights, points)))
 
@@ -21,9 +25,9 @@ def test_lobatto(order):
     points, weights = quadrature.lobatto(order + 1)
 
     x = sympy.Symbol("x")
-    poly = x ** (2 * order - 1)
+    poly = ScalarFunction(x ** (2 * order - 1))
 
-    assert symequal(
+    assert allequal(
         poly.integrate((x, 0, 1)),
         sum(i * poly.subs(x, j) for i, j in zip(weights, points)))
 
@@ -33,9 +37,9 @@ def test_radau(order):
     points, weights = quadrature.radau(order + 1)
 
     x = sympy.Symbol("x")
-    poly = x ** (2 * order - 1)
+    poly = ScalarFunction(x ** (2 * order - 1))
 
-    assert symequal(
+    assert allequal(
         poly.integrate((x, 0, 1)),
         sum(i * poly.subs(x, j) for i, j in zip(weights, points)))
 
@@ -45,8 +49,8 @@ def test_legendre(order):
     points, weights = quadrature.legendre(order)
 
     x = sympy.Symbol("x")
-    poly = x ** (2 * order - 1)
+    poly = ScalarFunction(x ** (2 * order - 1))
 
-    assert symequal(
+    assert allequal(
         poly.integrate((x, 0, 1)),
         sum(i * poly.subs(x, j) for i, j in zip(weights, points)))
